﻿using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using AddinExpress.MSO;
using System.Diagnostics;

namespace ReplyAllExplorer
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("B96EE51D-4017-4A03-940B-D540B7CA94CD"), ProgId("ReplyAllExplorer.AddinModule")]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }
        private ADXOutlookAppEvents adxOutlookEvents;
        private ReplyAllChecker replyAllChecker;


        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;

        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
            // 
            // adxOutlookEvents
            // 
            this.adxOutlookEvents.ExplorerActivate += new AddinExpress.MSO.ADXOlExplorer_EventHandler(this.adxOutlookEvents_ExplorerActivate);
            this.adxOutlookEvents.ExplorerSelectionChange += new AddinExpress.MSO.ADXOlExplorer_EventHandler(this.adxOutlookEvents_ExplorerSelectionChange);
            // 
            // AddinModule
            // 
            this.AddinName = "ReplyAllExplorer";
            this.HandleShortcuts = true;
            this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
            this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);
            this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);

        }
        #endregion

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
        {
            Debug.Print("The AddinStartupComplete event has occurred.");
            Debug.Print("The add-in has been loaded in Outlook " + this.HostMajorVersion.ToString());

            // This creates an instance of the class that handles the events of an Outlook item
            replyAllChecker = new ReplyAllChecker(this);
        }

        private void adxOutlookEvents_ExplorerSelectionChange(object sender, object explorer)
        {
            Debug.Print("The ExplorerSelectionChange event has occurred.");
            Outlook.Explorer theExplorer = explorer as Outlook.Explorer;
            if (theExplorer != null)
            {
                Outlook.Selection selection = null;
                try
                {
                    selection = theExplorer.Selection;
                }
                catch { }
                if (selection != null)
                {
                    ConnectToSelectedItem(selection);
                    Marshal.ReleaseComObject(selection);
                }
            }
        }

        private void AddinModule_AddinBeginShutdown(object sender, EventArgs e)
        {
            // this event may not occur in Outlook 2010, see the blog post 'Outlook 2010 Fast Shutdown feature'
            // at http://www.add-in-express.com/creating-addins-blog/2010/05/04/outlook2010-fast-shutdown/

            Debug.Print("The AddinBeginShutdown event has occurred.");

            if (replyAllChecker.IsConnected)
            {
                replyAllChecker.RemoveConnection();
                Debug.Print("Disconnected from the previously connected item.");
            }
            replyAllChecker = null;
        }

        private void adxOutlookEvents_ExplorerActivate(object sender, object explorer)
        {
            Debug.Print("The ExplorerActivate event has occurred.");
            Outlook.Explorer theExplorer = explorer as Outlook.Explorer;
            if (theExplorer != null)
            {
                Outlook.Selection selection = null;
                try
                {
                    selection = theExplorer.Selection;
                }
                catch { }
                if (selection != null)
                {
                    ConnectToSelectedItem(selection);
                    Marshal.ReleaseComObject(selection);
                }
            }
        }

        private void ConnectToSelectedItem(Outlook.Selection selection)
        {
            if (selection != null)
            {
                if (selection.Count == 1)
                {
                    object item = selection.Item(1);
                    if (item is Outlook.MailItem)
                    {
                        if (replyAllChecker.IsConnected)
                        {
                            replyAllChecker.RemoveConnection();
                            Debug.Print("Disconnected from the previously connected item.");
                        }
                        replyAllChecker.ConnectTo(item, true);
                        Debug.Print("Connected to this Outlook item.");
                    }
                    else
                    {
                        Marshal.ReleaseComObject(item);
                        Debug.Print("Do not connect to this Outlook item.");
                    }
                }
            }
        }
    }
}

